import { BaseRDFReader, RDFS } from "../BaseRDFReader";
import ISpecificationEntry from "../ISpecificationEntry";
import { Quad, Store } from "n3";
import { OWLSpecificationProvider } from "./OWLSpecificationProvider";
import { Config } from "../../ontologies/SparnaturalConfig";
import factory from "@rdfjs/data-model";

export class OWLSpecificationEntry extends BaseRDFReader implements ISpecificationEntry {
    uri:string;
    provider:OWLSpecificationProvider;


    constructor(uri:string, provider: OWLSpecificationProvider, n3store: Store<Quad>, lang: string) {
        super(n3store, lang);
        this.uri=uri;
        this.provider=provider;
    }

    getId(): string {
        return this.uri;
    }

    getLabel(): string {
        return this.#_readLabel(this.uri, this.lang);
    }

    getTooltip(): string {
        return this._readAsLiteralWithLang(this.uri, Config.TOOLTIP, this.lang);
    }

    getIcon(): string {
        var faIcon = this._readAsLiteral(
            this.uri,
            factory.namedNode(Config.FA_ICON)
          );
          if (faIcon.length > 0) {
            // use of fa-fw for fixed-width icons
            return (
              "<span style='font-size: 170%;' >&nbsp;<i class='" +
              faIcon +
              " fa-fw'></i></span>"
            );
          } else {
            var icons = this._readAsLiteral(this.uri, factory.namedNode(Config.ICON));
            if (icons.length > 0) {
              return icons[0];
            } else {
              // this is ugly, just so it aligns with other entries having an icon
              return "<span style='font-size: 175%;' >&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</span>";
            }
          }
    }

    getHighlightedIcon() {
      var icons = this._readAsLiteral(
        this.uri,
        factory.namedNode(Config.HIGHLIGHTED_ICON)
      );
      if (icons.length > 0) {
        return icons[0];
      } 
    }


    /**
     * Reads config:order of an entity and returns it, or null if not set
     **/
    #_readOrder(uri: any) {
        return this._readAsSingleLiteral(uri, Config.ORDER);
    }

    /**
     * Reads config:order of an entity and returns it, or null if not set
     **/
    #_readLabel(uri: any, lang:string) {
        return this._readAsLiteralWithLang(uri, RDFS.LABEL, lang);
    }
}